/*
  Version 3.0 03/2018
  Programme de gestion des PIR du Hall et Couloir
  Peu allumer la vitrine et le couloir
  1 fct :
    test si porte ouverte en premier puis pir hall
      allume la vitrine tant que presence devant pir
  2 fct :
    test si presence devant pir hall durant 60s
      allume la vitrine tant que presence devant pir
  3 fct :
    Test pir couloir si OK 
      allume lumiére 10s puis test presence.
      On remplace le recepteur RF par un relais.
      Test de luminosité.
  4 fct :
    Test d'appuie sur BP
      Vitrine allumée -> on eteint
      sinon on allume.
        
*/


// #include <RCSwitch.h>
#include "HRCSwitch.h"
HRCSwitch mySwitch = HRCSwitch();

// RCSwitch mySwitch = RCSwitch();

#define RX_PIN 		  2	// The receiver pin.
#define Con         5201  // Allume vitrine    
#define Coff        5204  // eteint vitrine
#define pir_hall    5592405  //pir hall
#define pir_couloir 5264467 //pir couloir
#define Acouloir    5558768   // Commande d'allumage du couloir
#define ctc_porte   5510992  //contact porte d'entrée.
#define bp_vit      70960    // Bp pour allumer/eteindre vitrine
#define tempsexec   10000 // Temps d'exécution 20s.
#define tempscoul   4000  // Temps de présence dans le couloir
#define bouton      9 // pin du bouton b0
#define LEDR        13  // LED used to blink when volume too high
#define LEDV        12  // LED used to blink when volume too high
#define lum_ambiante 920  // Lumière ambiante -> 1000 si plus sombre
#define lum_allume   750  // Lumiére allumée -> 0 si plus lumineux
#define relais1   8   //relais

// ----- Variables -----

boolean porte = false;  // false si la porte n'a pas été ouverte.
boolean hall = false;   // false si le pir du hall n'a pas été activé.
boolean couloir = false;  // false si pir du couloir n'a pas été activé.
boolean bp = false;     // false si bp non actif
boolean bp_flag = false;  // inverse la commande à l'appuie du bp
boolean RF = false;     // Si Rf présente
boolean vitrine = false;  //flag pour ne pas allumer plusieurs fois la vitrine
boolean avitrine = false;  //commande pour allumer la vitrine true=on allume, false=on eteint.
boolean presenceH = false;  // flag pour déterminer la présence dans le hall
boolean presenceC = false;  // flag pour déterminer la présence dans le couloir
boolean hallP = false;  // flag pour valider une détection du pir hall
boolean tallume = false; // flag pour ne pas remettre le temps à 0
boolean entree = false;  // pour dire que l'on sort.
long temps = 0;        // Temps d'exécution
long tempsP = 0;        // Temps de présence
long tempsA = 0;        // Temps avant d'allumer
long tempsC = 0;        // Temps de présence dans le couloir
unsigned long value = 0;  //retour RF
int   port = A0;    // port pour photoresistance
int   valeur = 0;         // Valeur du port A0
float vin = 0;          // Valeur en volt

void setup() {
  digitalWrite(relais1, HIGH);
  Serial.begin(115200);
  mySwitch.enableReceive(0);  	// Receiver on inerrupt 0 => that is pin #2
  mySwitch.enableTransmit(10);  // Using Pin #10
  pinMode(bouton, INPUT);        // button
  pinMode(LEDR, OUTPUT);       // sets the ledPin to be an output
  pinMode(LEDV, OUTPUT);
  pinMode(relais1, OUTPUT);     // initialise le port pour le relais.
  Serial.println("Hello ! C'est parti ...");
  
}

void rfEvent() {             // Test si nouveau signal RF
  porte=false;
  hall=false;
  couloir=false;
  bp=false;

if (mySwitch.available()) {
  value = mySwitch.getReceivedValue();
  RF = true;          // J'ai une valeur
  switch (value) {
  case ctc_porte: 
    porte = true;
  break;
  case pir_hall:
    hall = true;
  break;
  case pir_couloir:
    couloir = true;
    valeur = analogRead(port);
    // convertit l’entrée en volt 
    vin = (valeur * 5.0) / 1024.0;
  break;
  case Con: 
    vitrine = true;
  break;
  case Coff: 
    vitrine = false;
  break;
  case bp_vit: 
    bp = true;
  break;
  } 
} else {
  RF = false;
}

} // Fin du void


void loop() {
  temps = 0;
  rfEvent();

  if (RF) {      
    Serial.print("Reçu ");
    Serial.println(value); 
    Serial.print("hall ");
    Serial.println(hall); 

    if (bp) { 
      if (!bp_flag) {
        bp_flag = true;            // Si appui sur BP et pas allumé, on allume la vitrine
        mySwitch.send(Con, 24);       // On allume
        Serial.println("ALLUME VITRINE");
        vitrine = true;
      } else {
        bp_flag = false;            // Si appui sur BP et allumé, on éteins la vitrine
        mySwitch.send(Coff, 24);       // On allume
        Serial.println("ETEINT VITRINE");
        vitrine = false;
      } 
      mySwitch.resetAvailable();
    }
    
    if ((porte) ) {         // test si contact de porte 
      entree = true;                 // oui on valide le flag
      temps = millis();             // On stock le temps actuel pour voir si on est dans les délais
      mySwitch.send(Con, 24);       // On allume
      Serial.println("ALLUME VITRINE");
      delay(1000);                  // Delais pour fermer la porte.
      mySwitch.resetAvailable();
    }
    
    if ((hall) && (!vitrine)) {      // Si on est dans le hall sans la vitrine allumé
      hallP = true;
      if (!presenceH) {              // On active le temps de présence min         
        presenceH = true;
        tempsP=millis(); 
      
        if (!tallume) {               // On active le temps d'attente devant la vitrine
          tallume = true;
          tempsA=millis();
        }
        mySwitch.resetAvailable();
        if (entree)  {                 // Si on avait la porte ouverte avant
          Serial.println("entre");
          avitrine = true;            // C'est qu'on entre, on allume la vitrine
          //mySwitch.send(Con, 24);       // On allume
          //mySwitch.send(5201, 24);   // On allume 
          Serial.println("ALLUME VITRINE");
          mySwitch.send(Con, 24);       // On allume
          vitrine = true;               // on valide vitrine
          entree = false;               // on RAZ variable suivante
          tallume = false;
        }
      hall=false;
      }
      mySwitch.resetAvailable();
    }
    if (couloir) {
      Serial.print("valeur ");Serial.println(valeur);
      if (!presenceC) {              // On active le temps de présence min
        if (valeur >= lum_ambiante) {
        //  Serial.print("valeur ");Serial.print(valeur);
          presenceC = true;
          tempsC=millis();
          mySwitch.resetAvailable();
          digitalWrite(relais1, LOW);
          digitalWrite(LEDV, HIGH);
          Serial.println("ALLUME couloir");
        }
      }
      mySwitch.resetAvailable();
    }
    
  } 
  
  if (presenceH) {                 // Si on et en detection de presence
    if ((tallume) && (millis() - tempsA > 60000)) {    // On vérifie que c'est juste la presence dans le hall
      avitrine = true;            // Si le temps de presence dépasse les 1min30 on allume
      mySwitch.send(Con, 24);       // On allume
        Serial.println("ALLUME VITRINE");
      vitrine = true;
      tallume = false;            // On passe en mode présence 
      tempsA = 0;     
    }
    if ((hall) && (millis() - tempsP < tempsexec)) {    // hall sans les 20s de passé ?
      Serial.print("temps passé :");
      Serial.println(millis() - tempsP);
      tempsP=millis();    // oui on réinitialise le temps
    } 
    if ((!hall) && (millis() - tempsP > tempsexec)) {
      Serial.println(millis() - tempsP);
      presenceH = false;                 // Sinon y a plus personne
      tallume = false;
      tempsP = 0;
      mySwitch.send(Coff, 24);      // On éteint
    Serial.println("ETEINT vitrine");
      vitrine = false;
      if (avitrine) avitrine = false;   // on demande l'extinction
    }
  hall=false;
  }  

  if (presenceC) {
    if ((couloir) && (millis() - tempsC < tempscoul)) {    // 20s de passé ?
    //  Serial.print("temps passé couloir :");
    //  Serial.println(millis() - tempsC);
      tempsC=millis();                    // oui on réinitialise le temps
    } 
    if ((!couloir) && (millis() - tempsC > tempscoul)) {
      presenceC = false;
      couloir = false;
      tempsC = 0;
      digitalWrite(relais1, HIGH);
      digitalWrite(LEDV, LOW);
    }
    mySwitch.resetAvailable();
    value=0;
  }

}
